@@ -5,14 +5,15 @@
 #ifndef _INPUT_H
 #define _INPUT_H
 
-#include <SDL.h>
+#include "SDL_compat.h"
 
-#define HOTKEYS_MOD_MASK	(KMOD_SHIFT | KMOD_CTRL | KMOD_ALT | KMOD_META | KMOD_MODE)
+#define HOTKEYS_MOD_MASK	(KMOD_SHIFT | KMOD_CTRL | KMOD_ALT | KMOD_GUI | KMOD_MODE)
 
 void InputInit();
 void InputReset();
 void InputExit();
-bool grabMouse(bool grab);
+SDL_bool grabMouse(SDL_bool grab);
+SDL_bool hideMouse(SDL_bool grab);
 void check_event();
 
 #ifdef SDL_GUI
