@@ -26,6 +26,13 @@
 
 #include "natfeats.h"	/* nf_getparameter is defined there */
 
+#define DriveFromLetter(d) \
+	(((d) >= 'A' && (d) <= 'Z') ? (d - 'A') : \
+	 ((d) >= 'a' && (d) <= 'z') ? (d - 'a') : \
+	 ((d) >= '0' && (d) <= '5') ? (d - '0' + 26) : \
+	 -1)
+#define DriveToLetter(d) ((d) < 26 ? 'A' + (d) : (d) - 26 + '0')
+
 class NF_Base
 {
 public:
@@ -36,7 +43,7 @@ public:
 	virtual bool isSuperOnly() = 0;
 	virtual int32 dispatch(uint32 fncode) = 0;
 	uint32 getParameter(int i) { return nf_getparameter(i); }
+	uint32 errnoHost2Mint( int unixerrno,int defaulttoserrno );
 };
 
 #endif /* _NF_BASE_H */
-
