@@ -25,6 +25,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <termios.h>
+#ifdef __CYGWIN__
+#include <sys/socket.h> /* for FIONREAD */
+#endif
 
 #include "serial.h"
 #include "serial_port.h"
@@ -38,6 +41,8 @@
 Serialport::Serialport(void)
 {
 	D(bug("Serialport: interface created"));
+	oldTBE = 0;
+	oldStatus = 0;
 	handle = open(bx_options.serial.serport,O_RDWR|O_NDELAY|O_NONBLOCK);/* Raw mode */
 	// /dev/ttyS0 by default or /dev/ttyUSB0 : see Serport in ./aranym/config [SERIAL]
 	if (handle<0) {
@@ -215,9 +220,9 @@ uint16 Serialport::getStatus()
 uint16 Serialport::getTBE() // not suited to serial USB
 {
 	uint16 value=0;
-	int status=0;
 	
 #if defined(TIOCSERGETLSR) && defined(TIOCSER_TEMT)
+	int status=0;
 	if (ioctl(handle,TIOCSERGETLSR,&status)<0){// OK with ttyS0, not OK with ttyUSB0
 		//D(bug("Serialport: Can't get LSR"));
 		value|=(1<<TBE);// only for serial USB
