@@ -51,12 +51,13 @@ static unsigned int mmu_atc_hits[ATC_L2_SIZE];
 static void mmu_dump_ttr(const char * label, uae_u32 ttr)
 {
 	DUNUSED(label);
+#if DEBUG
 	uae_u32 from_addr, to_addr;
 
 	from_addr = ttr & MMU_TTR_LOGICAL_BASE;
 	to_addr = (ttr & MMU_TTR_LOGICAL_MASK) << 8;
 
-	D(bug("%s: [%08lx] %08lx - %08lx enabled=%d supervisor=%d wp=%d cm=%02d",
+	D(bug("%s: [%08x] %08x - %08x enabled=%d supervisor=%d wp=%d cm=%02d",
 			label, ttr,
 			from_addr, to_addr,
 			ttr & MMU_TTR_BIT_ENABLED ? 1 : 0,
@@ -64,6 +65,9 @@ static void mmu_dump_ttr(const char * label, uae_u32 ttr)
 			ttr & MMU_TTR_BIT_WRITE_PROTECT ? 1 : 0,
 			(ttr & MMU_TTR_CACHE_MASK) >> MMU_TTR_CACHE_SHIFT
 		  ));
+#else
+	DUNUSED(ttr);
+#endif
 }
 
 void mmu_make_transparent_region(uaecptr baseaddr, uae_u32 size, int datamode)
@@ -141,7 +145,7 @@ static void mmu_dump_table(const char * label, uaecptr root_ptr)
 	uaecptr ptr_des_addr, page_addr,
 		root_log, ptr_log, page_log;
 
-	D(bug("%s: root=%lx", label, root_ptr));
+	D(bug("%s: root=%x", label, root_ptr));
 
 	for (root_idx = 0; root_idx < ROOT_TABLE_SIZE; root_idx++) {
 		root_des = phys_get_long(root_ptr + root_idx);
@@ -309,6 +313,7 @@ static void mmu_bus_error(uaecptr addr, int fc, int write, int size)
 
 	D(bug("BUS ERROR: fc=%d w=%d log=%08x ssw=%04x", fc, write, addr, ssw));
 
+	breakpt();
 	THROW(2);
 }
 
@@ -441,7 +446,10 @@ uaecptr mmu_translate(uaecptr addr, int super, int data, int write)
 	l = &atc_l2[super][ATC_L2_INDEX(addr)];
 	mmu_fill_atc_l2(addr, super, data, write, l);
 	if (!(data ? l->valid_data : l->valid_inst))
+	{
+		breakpt();
 		THROW(2);
+	}
 
 	return addr + l->phys;
 }
@@ -539,6 +547,7 @@ uae_u16 mmu_get_word_unaligned(uaecptr addr, int data)
 		RESTORE_EXCEPTION;
 		regs.mmu_fault_addr = addr;
 		regs.mmu_ssw |= MMU_SSW_MA;
+		breakpt();
 		THROW_AGAIN(prb);
 	}
 	return res;
@@ -559,6 +568,7 @@ uae_u32 mmu_get_long_unaligned(uaecptr addr, int data)
 			RESTORE_EXCEPTION;
 			regs.mmu_fault_addr = addr;
 			regs.mmu_ssw |= MMU_SSW_MA;
+			breakpt();
 			THROW_AGAIN(prb);
 		}
 	} else {
@@ -574,6 +584,7 @@ uae_u32 mmu_get_long_unaligned(uaecptr addr, int data)
 			RESTORE_EXCEPTION;
 			regs.mmu_fault_addr = addr;
 			regs.mmu_ssw |= MMU_SSW_MA;
+			breakpt();
 			THROW_AGAIN(prb);
 		}
 	}
@@ -659,6 +670,7 @@ REGPARAM2 void mmu_put_long_unaligned(uaecptr addr, uae_u32 val, int data)
 			regs.mmu_fault_addr = addr;
 			regs.mmu_ssw |= MMU_SSW_MA;
 		}
+		breakpt();
 		THROW_AGAIN(prb);
 	}
 }
@@ -678,6 +690,7 @@ REGPARAM2 void mmu_put_word_unaligned(uaecptr addr, uae_u16 val, int data)
 			regs.mmu_fault_addr = addr;
 			regs.mmu_ssw |= MMU_SSW_MA;
 		}
+		breakpt();
 		THROW_AGAIN(prb);
 	}
 }
@@ -768,6 +781,7 @@ uae_u32 sfc_get_long(uaecptr addr)
 			RESTORE_EXCEPTION;
 			regs.mmu_fault_addr = addr;
 			regs.mmu_ssw |= MMU_SSW_MA;
+			breakpt();
 			THROW_AGAIN(prb);
 		}
 	} else {
@@ -783,6 +797,7 @@ uae_u32 sfc_get_long(uaecptr addr)
 			RESTORE_EXCEPTION;
 			regs.mmu_fault_addr = addr;
 			regs.mmu_ssw |= MMU_SSW_MA;
+			breakpt();
 			THROW_AGAIN(prb);
 		}
 	}
@@ -808,6 +823,7 @@ uae_u16 sfc_get_word(uaecptr addr)
 		RESTORE_EXCEPTION;
 		regs.mmu_fault_addr = addr;
 		regs.mmu_ssw |= MMU_SSW_MA;
+		breakpt();
 		THROW_AGAIN(prb);
 	}
 	return res;
@@ -848,6 +864,7 @@ void dfc_put_long(uaecptr addr, uae_u32 val)
 			regs.mmu_fault_addr = addr;
 			regs.mmu_ssw |= MMU_SSW_MA;
 		}
+		breakpt();
 		THROW_AGAIN(prb);
 	}
 }
@@ -874,6 +891,7 @@ void dfc_put_word(uaecptr addr, uae_u16 val)
 			regs.mmu_fault_addr = addr;
 			regs.mmu_ssw |= MMU_SSW_MA;
 		}
+		breakpt();
 		THROW_AGAIN(prb);
 	}
 }
@@ -891,6 +909,7 @@ void dfc_put_byte(uaecptr addr, uae_u8 val)
 	CATCH(prb) {
 		RESTORE_EXCEPTION;
 		regs.wb3_data = val;
+		breakpt();
 		THROW_AGAIN(prb);
 	}
 }
